--lua入口
-----------------------------------------------------------------------
function clone(object)
    local lookup_table = {};
    local function _copy(object)
        if type(object) ~= "table" then
            return object;
        elseif lookup_table[object] then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for key, value in pairs(object) do
            new_table[_copy(key)] = _copy(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _copy(object);
end

-- classCountList = {};
--Create an class.
function class(classname, super)
    local superType = type(super);
    local cls;

    if superType ~= "function" and superType ~= "table" then
        superType = nil;
        super = nil ;
    end

    cls = {};

    -- cls.__create = super

    cls.ctor    = function() end
    cls.__cname = classname;
    cls.__ctype = 1;

    local superClassName = "";

    function cls.new(...)
        -- local instance = cls.__create(...)
        local instance = super(...);
        if(instance.__cname ~= nil)then
            superClassName = instance.__cname;
        end
        -- if(classCountList[classname] == nil)then
        --     classCountList[classname] = 1;
        -- else
        --     classCountList[classname] = classCountList[classname] + 1;
        -- end
        -- copy fields from class to native object
        for k,v in pairs(cls) do
            if(type(v) == "function" and instance[k] ~= nil)then
                instance[superClassName.."_"..k] = instance[k];
                instance[k] = v;
            else
                instance[k] = v;
            end
        end
        instance.class = cls;
        instance:ctor(...);
        return instance;
    end

    return cls;
end

local setmetatableindex_ = function(t, index) 
    local mt = getmetatable(t)
    if not mt then mt = {} end
    if not mt.__index then
        mt.__index = index
        setmetatable(t, mt)
    elseif mt.__index ~= index then
        setmetatableindex_(mt, index)
    end
end
setmetatableindex = setmetatableindex_

function class2(classname, ...)
    local cls = {__cname = classname}

    local supers = {...}
    for _, super in ipairs(supers) do
        local superType = type(super)
        assert(superType == "nil" or superType == "table",
            string.format("class() - create class \"%s\" with invalid super class type \"%s\"",
                classname, superType))
        if super[".isclass"] then
            -- super is native class
            assert(cls.__create == nil,
                string.format("class() - create class \"%s\" with more than one creating function or native class",
                    classname));
            cls.__create = function() return super:create() end
        else
            -- super is pure lua class
            cls.__supers = cls.__supers or {}
            cls.__supers[#cls.__supers + 1] = super
            if not cls.super then
                -- set first super pure lua class as class.super
                cls.super = super
            end
        end
    end

    cls.__index = cls
    if not cls.__supers or #cls.__supers == 1 then
        setmetatable(cls, {__index = cls.super})
    end

    if not cls.ctor then
        -- add default constructor
        cls.ctor = function() end
    end
    cls.new = function(_,...)
        local instance
        if cls.__create then
            instance = cls.__create(...)
        else
            instance = {}
        end
        setmetatableindex(instance, cls)
        instance.class = cls
        instance:ctor(...)
        return instance
    end
    cls.create = function(_, ...)
        return cls.new(...)
    end

    return cls
end


-----------------------------------------------------------------------

local json = require "cjson";
local gameSDK;
local gameConfig;
local gameScene;
local isStart = false;
globalLoadingView = nil;
fileLoaderManager = nil;
memoryCount = 0;
memoryTotal = 0;
memoryCountStr = "";

function Start(gameObject)
    if jit and jit.off then jit.off() end
    math.randomseed(os.time());
    memoryTotal = collectgarbage("count") / 1024;
    memoryCount = memoryTotal;
    memoryCountStr = memoryCountStr .. "Game.lua:Start  " .. tostring(memoryCount) .. "  " .. tostring(memoryTotal) .. "\n";
    gameSDK = GameSDK:new(gameObject, json);
    local version = gameSDK:getVersion();
    if LuaFramework.AppConst.PlayerMode == 1 then
        LuaFramework.Util.AddResSearchPath(LuaFramework.AppConst.PcResPath);
    elseif LuaFramework.AppConst.PlayerMode == 2 then
        LuaFramework.Util.AddResSearchPath(LuaFramework.AppConst.PcResPath);
    elseif LuaFramework.AppConst.PlayerMode == 3 then
        local versionPath = string.gsub(version, "%.", "_");
        --版本目录
        LuaFramework.Util.AddResSearchPath(UnityEngine.Application.persistentDataPath .. "/" .. versionPath .. "/");
        --可读写目录
        LuaFramework.Util.AddResSearchPath(UnityEngine.Application.persistentDataPath .. "/");
        --APP目录
        LuaFramework.Util.AddResSearchPath(UnityEngine.Application.streamingAssetsPath .. "/");
    end
    
    require "GameConfig"
    gameConfig = GameConfig:new();
    gameConfig.bigVersion = version;
    if gameConfig:isAndroid() == true or gameConfig:isIOS() == true then
        --设置sdkId
        gameConfig.sdkId = tonumber(gameSDK:getSdkId());
    end

    require "LoadingView"
    local go = UnityEngine.GameObject.Find("loadingGo");
    globalLoadingView = LoadingView:new(go);

    fileLoaderManager = FileLoaderManager.New(UnityEngine.Application.persistentDataPath);
    -- checkDel(gameObject);

    checkVer(gameObject);
end

function checkVer(gameObject)
    fileLoaderManager:LoadWWWText(gameConfig:getCheckVerUrl(), function(t1, t2, txt)
        if txt ~= nil and string.len(txt) > 0 then
            local data = json.decode(txt);
            if data ~= nil then
                globalLoadingView:updateLoading(5);
                gameConfig.isShenhe = data.isShenhe == "1";
                gameConfig.configUrl = data.configUrl;
                if gameConfig.isShenhe == true then
                    LuaFramework.Util.Log(tostring(os.time()));
                    LuaFramework.Util.Log = function() end
                end
                LuaFramework.Util.Log("### checkVer:  "..txt);
                LuaFramework.Util.Log("### isShenhe: "..tostring(gameConfig.isShenhe).."   configUrl: "..tostring(gameConfig.configUrl));
                -- gameConfig.delList = json.decode(data.delList);
                -- gameConfig.urlList = json.decode(data.urlList);
                gameConfig.versionList = json.decode(data.versionList);


                local delStr = data.delList;
                local delList = {};
                local vals = string.split(delStr, "\r\n");
                if vals ~= nil and #vals > 0 then
                    for i=1, #vals do
                        local ps = string.split(vals[i], "|");
                        if ps ~= nil and #ps == 2 then
                            delList[ps[1]] = ps[2];
                        end
                    end
                end
                gameConfig.delList = delList;

                local urlStr = data.urlList;
                local urlList = {};
                local vals = string.split(urlStr, "\r\n");
                if vals ~= nil and #vals > 0 then
                    for i=1, #vals do
                        local ps = string.split(vals[i], "###");
                        if ps ~= nil and #ps == 2 then
                            urlList[ps[1]] = ps[2];
                        end
                    end
                end
                gameConfig.urlList = urlList;

                checkDel(gameObject);
            else
                LuaFramework.Util.Log("### checkVer加载错误");
                checkVer(gameObject);
            end
        else
            LuaFramework.Util.Log("### checkVer加载失败");
            checkVer(gameObject);
        end
    end);
end

function checkDel(gameObject)
    --缓存代码清理
    globalLoadingView:updateLoading(10);
    --local codeVersion = UnityEngine.PlayerPrefs.GetInt("codeVersion", 1);
    local str = gameSDK:readKV("codeVersion");
    local codeVersion = 1;
    if str ~= "" then codeVersion = tonumber(str);end
    LuaFramework.Util.Log("### codeVersion：  "..codeVersion);
    if codeVersion > 0 then
        local fileList;
        for k,v in pairs(gameConfig.delList) do
            if tonumber(k) == codeVersion then
                fileList = string.split(v, ",");
            end
        end
        if fileList ~= nil and #fileList > 0 then
            delFiles(fileList);
            --UnityEngine.PlayerPrefs.SetInt("codeVersion",0);
            gameSDK:writeKV("codeVersion", 0);
        end
    end

    enterGame(gameObject);
end

function delFiles(fileList)
    local versionPath = string.gsub(gameConfig.bigVersion, "%.", "_");
    for i=1, #fileList do
        if fileList[i] ~= "" then
            local versionFile = UnityEngine.Application.persistentDataPath .. "/" .. versionPath .. "/" .. fileList[i];
            if delFile(versionFile) == false then
                 versionFile = UnityEngine.Application.persistentDataPath .. "/" .. versionPath .. "/gamelua/" .. fileList[i];
				if delFile(versionFile) == false then
					versionFile = UnityEngine.Application.persistentDataPath .. "/" .. fileList[i];
					if delFile(versionFile) == false then
						versionFile = UnityEngine.Application.persistentDataPath .. "/gamelua/" .. fileList[i];
						delFile(versionFile);
					end
				end
            end
        end
    end
end
function delFile(path)
	local isExists = System.IO.File.Exists(path);
	if isExists == true then
        LuaFramework.Util.Log("### 删除文件："..path);
        System.IO.File.Delete(path);
    end
    return isExists;
end

function enterGame(gameObject)
    if(isStart == true)then return; end
    globalLoadingView:updateLoading(15);
    isStart = true;
    require "GameScene"
    gameScene = GameScene:new(gameObject, gameConfig, gameSDK, json);
end

function cshapeCallLua(type,param1,param2,param3,param4,param5)
	if gameScene ~= nil then
		gameScene:cshapeCallLua(type,param1,param2,param3,param4,param5);
	end
    if type == 1 and globalLoadingView ~= nil then
        globalLoadingView:tick();
    end
end

function callToUnity(data)
    gameSDK:callToUnity(data);
end


GameSDK = class("GameSDK", function (_, gameObj, json)
    local self = {};
    self.gameObj = gameObj;
    self.json = json;
    self.sdk = self.gameObj:GetComponent(typeof(LuaFramework.SDKManager));
    self.handlers = {};
    return self;
end)

function GameSDK:ucall( code, data )
    return self.sdk:ucall( code, data );
end

function GameSDK:callToUnity( data )
    print("GameSDK:callToUnity", data);
    LuaFramework.Util.Log("#GameSDK callToUnity： "..data);
    jdata = self.json.decode(data);
    self:doCallback(jdata);
end

function GameSDK:addCallback( code, callBack, cbTarget )
    local list = self.handlers[code]
    if(list == nil) then
        list = {};
        self.handlers[code] = list;
    end
    local index = self:contain( list, callBack, cbTarget );
    if(index == -1) then
        local cbObj = { cb = callBack, tar = cbTarget };
        table.insert(list, cbObj);
    end
    return self;
end

function GameSDK:removeCallback( code, callBack, cbTarget )
    local list = self.handlers[code];
    if(list ~= nil) then
        local index = self:contain( list, callBack, cbTarget );
        if(index > -1) then
            table.remove(list, index)
        end
    end
    return self;
end

function GameSDK:contain( list, callBack, cbTarget )
    for i = 1, #list do
        local item = list[i];
        if(item.cb == callBack and item.tar == cbTarget) then
            return i;
        end
    end
    return -1;
end

function GameSDK:doCallback( jdata )
    if jdata ~= nil and jdata.code ~= nil then
        local list = self.handlers[tonumber(jdata.code)];
        if(list ~= nil) then
            for i = 1, #list do
                local item = list[i];
                if item ~= nil then
                    item.cb(item.tar, jdata.data);
                end
            end
        end
    end
end

function GameSDK:getVersion()
    local version = self:ucall(4);
    if version == nil or version == "" then
        version = "1.0";
    end
    return version;
end

function GameSDK:getSdkId()
    local sdkId = self:ucall(5);
    if sdkId == nil or sdkId == "" then
        sdkId = "0";
    end
    return sdkId;
end

function GameSDK:initSDK(callBack, callBackTarget)
    local initSDKCallback = function(target, data)
        self:removeCallback(2000, initSDKCallback, nil);
        if callBack ~= nil then
            callBack(callBackTarget, data);
        end
    end

    self:addCallback(2000, initSDKCallback);
    self:ucall(2000);
end

function GameSDK:getNetworkState()
    return self:ucall(6);
end

function GameSDK:readKV(key)
    if UnityEngine.Application.isMobilePlatform == true then
        return self:ucall(203,key);
    else
        return UnityEngine.PlayerPrefs.GetString(key);
    end
end

function GameSDK:writeKV(key,value)
    if UnityEngine.Application.isMobilePlatform == true then
        local data={key = key,value = value};
        local jdata = self.json.encode(data);
        self:ucall(202,jdata);
    else
        local v = tostring(value);
        UnityEngine.PlayerPrefs.SetString(key,v);
    end
end

function GameSDK:sendData(data)
    self:ucall(2005, self.json.encode(data));
end

function GameSDK:startLock()
    self:ucall(104);
end

function GameSDK:releaseLock()
    self:ucall(105);
end

function GameSDK:restartGame()
    self:ucall(114);
end