--游戏更新初始流程
GameScene = class("GameScene",function(self, gameObj, gameConfig, gameSDK, json)
	local gs = {};
	gs.gameObj = gameObj;
	gs.gameStart = nil;
	gs.gameConfig = gameConfig;
	gs.updateManager = nil;
	gs.json = json;
	gs.gameSDK = gameSDK;
	--1：新人开发模式，2：开发模式，3：发布模式
	gs.codeType = 3;
	gs.luaAB = nil;
	return gs;
end)

function GameScene:ctor()
	--设置项目属性
	--将gameObject设置为不自动回收
	UnityEngine.Object.DontDestroyOnLoad(self.gameObj);
	--设定帧率
	UnityEngine.Application.targetFrameRate = 30;
	--关闭垂直同步
	if UnityEngine.QualitySettings.vSyncCount ~= 0 then
		UnityEngine.QualitySettings.vSyncCount = 0;
	end
	--永不进入暗屏模式和自动锁屏
	UnityEngine.Screen.sleepTimeout = UnityEngine.SleepTimeout.NeverSleep;
	self.gameSDK:startLock();
	UnityEngine.Shader.SetGlobalColor( "_Ambient", Color(0.7, 0.7, 0.7));

	local localIsDownload = self.gameSDK:readKV("local_isDownload");
	if localIsDownload == nil or localIsDownload == "" then
		self.gameSDK:writeKV("local_isDownload", tostring(self.gameConfig.isDownload));
	else
		self.gameConfig.isDownload = localIsDownload == "true";
	end
	LuaFramework.Util.Log("### localIsDownload: "..tostring(localIsDownload));
	LuaFramework.Util.Log("### isDownload: "..tostring(self.gameConfig.isDownload));

	if self.gameConfig:isNoSdkOrTestSdk() == true then
		self:startUpdate();
	else
		self:initGameSDK();
	end
end

function GameScene:initGameSDK()
	-- init SDK
	LuaFramework.Util.Log("### init SDK");
	globalLoadingView:updateLoading(20);
	self.gameSDK:initSDK(self.initSdkCompleted, self);
end

function GameScene:initSdkCompleted(data)
	if data == "true" then
		globalLoadingView:updateLoading(25);
		LuaFramework.Util.Log("### init SDK complete");
		self:startUpdate();
	else
		LuaFramework.Util.Log("### init SDK fail");
		if self.gameSDK:getNetworkState() ~= "1" then
			self:initGameSDK();
		end
	end
end

function GameScene:startUpdate()
	self.updateManager = GameUpdate:new(self);
	self.updateManager:initUrl();
	if self.gameConfig:needUpdate() == true then
		-- 启动更新
		LuaFramework.Util.Log("### check update");
		fileLoaderManager:SetUnCompressPath(UnityEngine.Application.persistentDataPath .. "/" .. string.gsub(self.gameConfig.bigVersion, "%.", "_") .. "/");
		self.updateManager:checkUpdate();
	else
		-- 无需更新直接进入游戏
		self:updateComplete();
	end
end

--更新完成，进入游戏
function GameScene:updateComplete()
	globalLoadingView:updateLoading(39);
	if(self.codeType == 2)then
		require "ImportClient"
	elseif(self.codeType == 1)then
		require "ImportClient2"
	end

	if(self.codeType == 1)then
		StartCoroutine(function() self:loadSourcelibs(); end);
	elseif(self.codeType == 2)then
		self:start();
	elseif(self.codeType == 3)then
		globalLoadingView:updateLoading(41);
		StartCoroutine(function() self:loadLuafile(); end);
	end
end

function GameScene:start()
	globalLoadingView:updateLoading(45);
	self.gameStart = GameStart:new();
	self.gameStart:setup(self);
end

function GameScene:clearUpdate()
end

function GameScene:cshapeCallLua(type,param1,param2,param3,param4,param5)
	if self.gameStart ~= nil then
		self.gameStart:cshapeCallLua(type,param1,param2,param3,param4,param5);
	end
	if(type == 1 and self.updateManager ~= nil)then
		self.updateManager:loadTick();
	end
end

--加载Sourcelibs
function GameScene:loadSourcelibs()
	local path = LuaFramework.Util.GetResSearchPath("sourcelibs");
	if self.gameConfig:isAndroid() == true or self.gameConfig:isIOS() == true then
		if string.find(path, UnityEngine.Application.persistentDataPath) ~= nil then
			path = "file://"..path;
		end
	end
	local download = UnityEngine.WWW(path);
	Yield(download);
	if(download.error ~= "" and download.error ~= nil)then
		print("sourcelibs加载出错了",download.error);
		return;
	else
		local asset = download.assetBundle:LoadAsset("sourcelibs.txt");
		local luaMgr = self.gameObj:GetComponent(typeof(LuaFramework.LuaManager));
		luaMgr:LuaDoString(asset.text);
	end
	download:Dispose();
	download = nil;

	self:start();
end

--加载luafile
function GameScene:loadLuafile()
	local path = LuaFramework.Util.GetResSearchPath("luafile");
	if self.gameConfig:isAndroid() == true then
		if string.find(path, UnityEngine.Application.persistentDataPath) ~= nil then
			path = "file://"..path;
		end
	end
	if self.gameConfig:isIOS() == true then
		path = "file://"..path;
	end
	LuaFramework.Util.Log("### loadLuafile      "..path);
	globalLoadingView:updateLoading(42);
	local download = UnityEngine.WWW(path);
	Yield(download);
	globalLoadingView:updateLoading(43);
	if(download.error ~= "" and download.error ~= nil)then
		print("#luafile加载出错了    "..download.error);
		return;
	else
		globalLoadingView:updateLoading(44);
		self.luaAB = download.assetBundle;
		local asset = self.luaAB:LoadAsset("targetLuaFile.txt");
		local luaMgr = self.gameObj:GetComponent(typeof(LuaFramework.LuaManager));
		luaMgr:LuaDoString(asset.text);
	end
	download:Dispose();
	download = nil;

	self:start();
end






GameUpdate = class("GameUpdate",function(self,gameScene)
	local gu = {};
	gu.gameScene = gameScene;
	gu.json = gameScene.json;
	gu._downUrls = {};
	gu._errorCount = 0;
	gu._updatePaths = {};
	gu._fileCount = 0;
	gu._currentItem = nil;
	gu._tryTime = 3;
	gu._loadFlag = false;
	gu._loadDelay = 0;
	gu._restartList = {};
	gu._needRestart = false;
	gu._miniVersion = 0;
	return gu;
end)

function GameUpdate:checkUpdate()
	-- LuaFramework.Util.Log("#load url.txt");
	-- self:loadTxtContent(self.gameScene.gameConfig:getVerUrl("url.txt"), self.loadUrlComplete, self);

	-- self:initUrl();
	if self.gameScene.gameConfig.isShenhe == false then
		self:checkVersion();
	else
		self:updateComplete();
	end
end

function GameUpdate:initUrl()
	LuaFramework.Util.Log("### GameUpdate  initUrl");
	globalLoadingView:updateLoading(30);
	local dict = self.gameScene.gameConfig.urlList;
	if dict ~= nil then
		local key = tostring(self.gameScene.gameConfig.sdkId);
		local url = dict[key];
		if url == nil then
			key = "0";
			url = dict[key];
		end
		if self.gameScene.gameConfig:isTestServer() then
			key = "-1";
			url = dict[key];
		end
		url = self:formatDownloadUrl(url);
		table.insert(self._downUrls,url);
		local newUrl = dict[tostring(tonumber(key) + 20000)];
		if newUrl ~= nil then
			newUrl = self:formatDownloadUrl(newUrl);
			table.insert(self._downUrls,newUrl);
		end
		self.gameScene.gameConfig.downUrls = self._downUrls;

		--资源包下载地址
		if self.gameScene.gameConfig.isDownload == true then
			if self.gameScene.gameConfig:isTestServer() then
				if dict["test"] ~= nil then
					self.gameScene.gameConfig.downloadUrl = self:formatDownloadUrl(dict["test"]);
				else
					self.gameScene.gameConfig.downloadUrl = self:formatDownloadUrl(self.gameScene.gameConfig.downloadUrl);
				end
			else
				if dict["pkgs"] ~= nil then
					self.gameScene.gameConfig.downloadUrl = self:formatDownloadUrl(dict["pkgs"]);
				else
					self.gameScene.gameConfig.downloadUrl = self:formatDownloadUrl(self.gameScene.gameConfig.downloadUrl);
				end
			end
			LuaFramework.Util.Log("### 资源包下载地址    "..self.gameScene.gameConfig.downloadUrl);
		end
	end
end

function GameUpdate:checkVersion()
	LuaFramework.Util.Log("### GameUpdate  checkVersion");
	globalLoadingView:updateLoading(35);
	local codes = {};
	local passVer = self.gameScene.gameConfig.resVersion;
	local currentVer = 0;
	local additionStr = "";
	local data = self.gameScene.gameConfig.versionList;
	for k,v in pairs(data) do
		if self.gameScene.gameConfig.isDownload == true then
			if string.find("cd", k) ~= nil then
				--代码 数据
				if self.gameScene.gameConfig:needUpdate() == true then
					currentVer = v[1];
					if passVer < currentVer then
						table.insert(codes,{key=k, version=currentVer, androidSize=tonumber(v[2]), iosSize=tonumber(v[3])});
					end
				end
			elseif k == "n" then
				--mini包版本
				local tmp = self.gameScene.gameSDK:readKV("miniVersion");
				if tmp ~= nil and tmp ~= "" then
					self._miniVersion = tonumber(tmp);
				end
				currentVer = v[1];
				if self._miniVersion < currentVer then
					table.insert(codes,{key=k, version=currentVer, androidSize=tonumber(v[2]), iosSize=tonumber(v[2])});
				end
			end
		else
			if string.find("cdfub", k) ~= nil then
				--代码 数据 字体 公共图集
				if self.gameScene.gameConfig:needUpdate() == true then
					currentVer = v[1];
					if passVer < currentVer then
						table.insert(codes,{key=k, version=currentVer, androidSize=tonumber(v[2]), iosSize=tonumber(v[3])});
					end
				end
			elseif k == "v" then
				--资源
				if self.gameScene.gameConfig:needUpdate() == true then
					for i=1, #v do
						currentVer = v[i][1];
						if passVer < currentVer then
							table.insert(codes,{key=k, version=currentVer, androidSize=tonumber(v[i][2]), iosSize=tonumber(v[i][3])});
						end
					end
				end
			elseif string.find("hijk", k) ~= nil then
				--增量包
				local plusList = {};
				for i=1, #v do
					table.insert(plusList,{version=v[i][1], androidSize=tonumber(v[i][2]), iosSize=tonumber(v[i][3])});
				end
				self.gameScene.gameConfig.plusList = plusList;
			elseif k == "r" then
				--需要重启游戏的版本
				for i=1, #v do
					table.insert(self._restartList, v[i]);
				end
			elseif k == "a" then
				--分包，进入游戏立即更新
				if self.gameScene.gameConfig:needAdditionUpdate() == true then
					local aPassVer = 0;
					local aVersion = 0;--读取本地a包版本
					if aVersion ~= nil and aVersion ~= "" then
						aPassVer = tonumber(aVersion);
					end
					for i=1, #v do
						currentVer = v[i][1];
						if passVer < currentVer then
							table.insert(codes,{key=k, version=currentVer, androidSize=tonumber(v[i][2]), iosSize=tonumber(v[i][3]), count=#v});
						end
					end
				end
			end
		end
	end
	table.sort(codes, function(a,b)
		if a.key == "a" and b.key == "a" then
			return a.version < b.version;
		elseif a.key == "a" then
			return true;
		elseif b.key == "a" then
			return false;
		else
			return a.version < b.version;
		end
	end);
	self._updatePaths = codes;
	self._fileCount = #self._updatePaths
	if self._fileCount > 0 then
		-- local size = 0;
		-- local isAndroid = self.gameScene.gameConfig:isAndroid();
		-- for i=1, #self._updatePaths do
		-- 	if isAndroid == true then
		-- 		size = size + self._updatePaths[i].androidSize;
		-- 	else
		-- 		size = size + self._updatePaths[i].iosSize;
		-- 	end
		-- end
		-- globalLoadingView:showAlert("更新内容大小为"..size.."M，点击确定开始下载。", self.updateCallback, self);
		self:updateCallback(1);
	else
		self:updateComplete();
	end
end

function GameUpdate:updateCallback(state)
	if state == 1 then
		LuaFramework.Util.Log("### 开始更新   资源版本："..self.gameScene.gameConfig.resVersion.."   mini版本："..self._miniVersion);
		self:loadNext();
	end
end

function GameUpdate:reDownload()
	if self._tryTime > 0 then
		self._tryTime = self._tryTime - 1;
		table.insert(self._updatePaths,1,self._currentItem);
		self:loadNext();
	end
end

function GameUpdate:loadNext()
	if self._updatePaths ~= nil and #self._updatePaths > 0 then
		globalLoadingView:updateLoading((self._fileCount-#self._updatePaths+1)/self._fileCount*100, "Downloading("..(self._fileCount-#self._updatePaths+1).."/"..self._fileCount..")");
		self._loadFlag = true;
		self._loadDelay = 5;
	else
		self._currentItem = nil;
		self:updateComplete();
	end
end

function GameUpdate:loadTick()
	if self._loadDelay > 0 then
		self._loadDelay = self._loadDelay - 1;
		return;
	end
	if(self._loadFlag == false)then return; end
	self._loadFlag = false;
	self._currentItem = table.remove(self._updatePaths,1);
	local url = self._downUrls[1];
	if self._errorCount % 2 == 1 and self._downUrls[2] ~= nil then
		url = self._downUrls[2];
	end
	local fileName = self._currentItem.key .. self._currentItem.version .. ".zip";
	local downloadFile = url  .. fileName;
	LuaFramework.Util.Log("### 开始下载    "..fileName.."    "..downloadFile);
	fileLoaderManager:DownloadFile(downloadFile,fileName,self.loadFileComplete,self,self.loadFileFail);
end

--code 1下载完成 2解压完成
function GameUpdate:loadFileComplete(self,code)
	if code == 2 then
		LuaFramework.Util.Log("### 解压完成    "..self._currentItem.key..self._currentItem.version);
		if self._currentItem.key == "c" then
			--UnityEngine.PlayerPrefs.SetInt("codeVersion", tonumber(self._currentItem.version));
			self.gameScene.gameSDK:writeKV("codeVersion", tonumber(self._currentItem.version));
		end
		if self._currentItem.key == "n" then
			self.gameScene.gameSDK:writeKV("miniVersion", tonumber(self._currentItem.version));
			self._miniVersion = tonumber(self._currentItem.version);
		end
		if string.find("cdfuv", self._currentItem.key) ~= nil then
			self.gameScene.gameConfig.resVersion = tonumber(self._currentItem.version);
		end
		for i=1, #self._restartList do
			if self._restartList[i] == self._currentItem.version then
				self._needRestart = true;
				break;
			end
		end
		self:loadNext();
	else
		LuaFramework.Util.Log("### 下载完成    "..self._currentItem.key..self._currentItem.version);
	end
end

function GameUpdate:loadFileFail(self,message)
	LuaFramework.Util.Log("### 加载失败    "..self._currentItem.key..self._currentItem.version .. ": " .. message);
	self._errorCount = self._errorCount + 1;
	self:reDownload();
end

function GameUpdate:updateComplete()
	-- self.gameScene.gameConfig.downUrls = self._downUrls;
	if self._needRestart == true then
		LuaFramework.Util.Log("### 重启游戏");
		fileLoaderManager = nil;
		UnityEngine.SceneManagement.SceneManager.LoadScene("RestartScene");
	else
		self.gameScene:updateComplete();
	end
end

function GameUpdate:formatDownloadUrl(url)
	url = self:fixUrl(url);
	if url ~= nil and string.len(url) > 0 then
		if string.sub(url,-1) ~= "/" then
			url = url .. "/";
		end
		if self.gameScene.gameConfig:isIOS() then
			url = url .. "ios/";
		elseif self.gameScene.gameConfig:isAndroid() then
			url = url .. "android/";
		end
	end
	return url;
end

function GameUpdate:fixUrl(url)
	if(url == nil)then return url; end
	local len = string.len(url);
	if(len <= 0)then return url; end
	local result = "";
	for i = 1,len do
		local charCode = string.byte(url,i);
		if(charCode ~= 13 and charCode ~= 32 and charCode ~= 160)then
			result = result..string.sub(url,i,i);
		end
	end
	return result;
end