LoadingView = class("LoadingView",
	function(_,go)
		local  obj = {};
		obj.gameObject = go;
		obj.transform = nil;
		obj._loadingViewTrans = nil;

		obj._bg = nil;--背景图
		obj._barLabel = nil;
		obj._bar = nil;
		obj._barTran = nil;
		obj._barHeight = 0;
		obj._currentValue = 0;
		obj._targetValue = 0;
		obj._tipContent = "";
		obj._totalW = 0;
		obj._delValue = 0;
		obj._frameCount = 15;

		obj._tipBoxTrans = nil;
		obj._tipBoxGo = nil;
		obj._sureBtn = nil;
		obj._cancelBtn = nil;
		obj._descriptLabel = nil;
		obj._callBack = nil;
		obj._cbTarget = nil;

		obj._isInMotion = false;

		obj._defaultStr = "正在启动游戏...%d%%";
		obj._specailStr = nil;
		obj._descriptStr = nil;
		return obj;
	end
);

function LoadingView:loadResource()
	local prefab = UnityEngine.Resources.Load("loadingView/loadingGo");
    if prefab ~= nil then
        self.gameObject = UnityEngine.GameObject.Instantiate(prefab);
        self.transform = self.gameObject.transform;
        UnityEngine.GameObject.DontDestroyOnLoad(self.gameObject);
    end
end


function LoadingView:ctor()
	self.transform = self.gameObject.transform;
	self.canvasTransform = self.transform:Find("loadingCanvas");
	local size = self.canvasTransform.sizeDelta;

    self._loadingViewTrans = self.transform:Find("loadingCanvas/loadingView");
	self._bg = self._loadingViewTrans:Find("bgImg"):GetComponent(typeof(UnityEngine.UI.Image));
	self._bg.transform.sizeDelta = Vector2(1280,800);
	self._barLabel = self._loadingViewTrans:Find("barLabel"):GetComponent(typeof(UnityEngine.UI.Text));
	self._bar = self._loadingViewTrans:Find("bar"):GetComponent(typeof(UnityEngine.UI.Image));
	self._barTran = self._bar.transform;
	self._barHeight = self._barTran.sizeDelta.y;
	self._barTran.sizeDelta = Vector2(1,self._barHeight);
	self._totalW = 1280;
	self._barLabel.text = "正在启动游戏...";

	self._tipBoxTrans = self._loadingViewTrans:Find("tipBox");
	self._tipBoxGo = self._tipBoxTrans.gameObject;
	self._sureBtn = self._tipBoxTrans:Find("sureBtn"):GetComponent(typeof(GameLib.ui.UIButton));
	self._cancelBtn = self._tipBoxTrans:Find("cancelBtn"):GetComponent(typeof(GameLib.ui.UIButton));
	self._descriptLabel = self._tipBoxTrans:Find("desLabel"):GetComponent(typeof(GameLib.ui.UIText));
	self._sureBtn.onPointerClick = function()self:onSureBtnClick();end
	self._cancelBtn.onPointerClick = function()self:onCancelBtnClick();end
	self:addEventSystem();
end

function LoadingView:addEventSystem()
	local go = UnityEngine.GameObject("EventSystem");
	go.transform:SetParent(self.transform);
    self._eventSystem = go:AddComponent(typeof(UnityEngine.EventSystems.EventSystem));
    self._eventSystem.sendNavigationEvents = false;
    self._eventSystem.pixelDragThreshold = 15;
    go:AddComponent(typeof(UnityEngine.EventSystems.StandaloneInputModule));
end

--streamasset目录,游戏内加载背景图
function LoadingView:loadGameBg()
	local path = LuaFramework.Util.GetResSearchPath("loading2.jpg");
	if UnityEngine.Application.platform == UnityEngine.RuntimePlatform.Android then
		if string.find(path, UnityEngine.Application.persistentDataPath) ~= nil then
			path = "file://"..path;
		end
	end
	if UnityEngine.Application.platform == UnityEngine.RuntimePlatform.IPhonePlayer then
		path = "file://"..path;
	end
	StartCoroutine(function() self:__load(path); end)
end

function LoadingView:__load(path)
	LuaFramework.Util.Log("loadingBgPath:"..path);
	local download = UnityEngine.WWW(path);
	Yield(download);
	if download.error ~= "" and download.error ~= nil then
		--加载出错
		LuaFramework.Util.Log("loadingbg加载出错");
	else
		LuaFramework.Util.Log("loadingbg加载成功");
		local asset = download.texture;
		self._bg.sprite = UnityEngine.Sprite.Create(asset,UnityEngine.Rect(0,0,asset.width,asset.height),Vector2.zero);
	end
end

--更改背景图
function LoadingView:resetBg(sprite)
	self._bg.sprite = sprite;
end

--预留需要更换loadingView的时候用
function LoadingView:resetGo(go)

end

function LoadingView:showLoading()
	self:_show();
end

function LoadingView:setDescript(str)
	self._descriptStr = str;
end

function LoadingView:updateLoading(progress,tipStr)
	self:_updateProgress(progress);
	self._specailStr = tipStr;
end

function LoadingView:hideLoading()
	self:_hide();
	self._descriptStr = nil;
end

function LoadingView:showAlert(message,callBack,cbTarget)
	self._tipBoxGo:SetActive(true);
	self._descriptLabel.text = message;
	self._callBack = callBack;
	self._cbTarget = cbTarget;
end

function LoadingView:onSureBtnClick()
	self._tipBoxGo:SetActive(false);
	if self._callBack ~= nil then
		self._callBack(self._cbTarget,1);
	end
end

function LoadingView:onCancelBtnClick()
	self._tipBoxGo:SetActive(false);
	if self._callBack ~= nil then
		self._callBack(self._cbTarget,0);
	end
end

function LoadingView:_show()
	self.gameObject:SetActive(true);
	if self._specailStr ~= nil then
		self._barLabel.text = string.format(self._specailStr,self._currentValue);
	elseif self._descriptStr ~= nil then
		self._barLabel.text = self._descriptStr .. string.format("..%d%%",self._currentValue);
	else
		self._barLabel.text = string.format(self._defaultStr,self._currentValue);
	end
end

function LoadingView:_updateProgress(progress)
	if progress < 0 then
		progress = 0;
	end
	if progress > 100 then
		progress = 100;
	end
	self._targetValue = progress;
	self._delValue = (self._targetValue - self._currentValue)/self._frameCount;
	self:_start();
end

function LoadingView:_start()
	self._isInMotion = true;
end

function LoadingView:_stop()
	self._isInMotion = false;
end

function LoadingView:tick(dt)
	if self._isInMotion == false then
		return;
	end
	self._currentValue = self._currentValue + self._delValue;
	if self._currentValue >= self._targetValue then
		self._currentValue = self._targetValue;
		self:_stop();
	end
	if self._specailStr ~= nil then
		self._barLabel.text = string.format(self._specailStr,self._currentValue);
	elseif self._descriptStr ~= nil then
		self._barLabel.text = self._descriptStr .. string.format("..%d%%",self._currentValue);
	else
		self._barLabel.text = string.format(self._defaultStr,self._currentValue);
	end

	local w = self._currentValue/100 * self._totalW;
	self._barTran.sizeDelta = Vector2(w,self._barHeight);
end

function LoadingView:_hide()
	self:_stop();
	self._barLabel.text = string.format(self._defaultStr,1);
	self._barTran.sizeDelta = Vector2(1,self._barHeight);
	self._targetValue = 0;
	self._currentValue = 0;
	self.gameObject:SetActive(false);
end